#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.utils import is_valid, _
from holmes.validators.base import Validator


class UrlWithUnderscoreValidator(Validator):

    @classmethod
    def get_url_with_underscore_message(cls):
        return _('Google treats a hyphen as a word separator, but does '
                 'not treat an underscore that way. Google treats and '
                 'underscore as a word joiner, so red_sneakers is the '
                 'same as redsneakers to Google. This has been confirmed '
                 'directly by Google themselves, including the fact that '
                 'using dashes over underscores will have a (minor) '
                 'ranking benefit.')

    @classmethod
    def get_violation_definitions(cls):
        return {
            'invalid.url_word_separator': {
                'title': _('URLs should use hyphens to separate words'),
                'description': cls.get_url_with_underscore_message(),
                'category': _('SEO'),
                'generic_description': cls.get_url_with_underscore_message()
            }
        }

    def validate(self):
        url = self.reviewer.page_url
        parsed_url = is_valid(url)
        path = parsed_url.path

        if '_' in path:
            self.add_violation(
                key='invalid.url_word_separator',
                value=url,
                points=10
            )



#!/usr/bin/python
# -*- coding: utf-8 -*-

from derpconf.config import Config  # NOQA
from holmes.utils import _


MINUTE = 60
HOUR = MINUTE * 60
DAY = 24 * HOUR

Config.define('WORKER_SLEEP_TIME', 10, _('Main loop sleep time'), 'Worker')
Config.define('ZOMBIE_WORKER_TIME', 200,
              _('Time to remove a Worker from API List (must be greater than WORKER_SLEEP_TIME + Validation time)'), 'API')

Config.define('WORKERS_LOOK_AHEAD_PAGES', 10000, _('Number of pages that will be retrieved when looking for the next job'), 'Worker')

Config.define('CONNECT_TIMEOUT_IN_SECONDS', 10, _('Number of seconds a connection can take.'), 'Worker')
Config.define('REQUEST_TIMEOUT_IN_SECONDS', 10, _('Number of seconds a request can take.'), 'Worker')

Config.define('HOLMES_API_URL', 'http://localhost:2368', ('URL that Worker will communicate with API'), 'Worker')
Config.define('HOLMES_WEB_URL', 'http://local.holmes.com:9000', _('URL of the client side application of Holmes'), 'Web')

Config.define('LOG_LEVEL', 'ERROR', 'Default log level', 'Logging')
Config.define('LOG_FORMAT', '%(asctime)s:%(levelname)s %(module)s - %(message)s',
              _('Log Format to be used when writing log messages'), 'Logging')
Config.define('LOG_DATE_FORMAT', '%Y-%m-%d %H:%M:%S',
              _('Date Format to be used when writing log messages.'), 'Logging')

Config.define('FACTERS', [], _('List of classes to get facts about a website'), 'Review')
Config.define('VALIDATORS', [], _('List of classes to validate a website'), 'Review')
Config.define('REVIEW_EXPIRATION_IN_SECONDS', 6 * 60 * 60, _('Number of seconds that a review expires in.'), 'Review')
Config.define('NUMBER_OF_REVIEWS_TO_KEEP', 4, _('Maximum number of reviews to keep'), 'Review')

Config.define('DAYS_TO_KEEP_REQUESTS', 12, _('Number of days to keep requests'), 'Requests')
Config.define('MAX_REQUESTS_FOR_FAILED_RESPONSES', 1000, _('Number of requests for failed responses'), 'Requests')

Config.define('MAX_ENQUEUE_BUFFER_LENGTH', 1000,
              _('Number of URLs to enqueue before submitting to the /pages route'), 'Validators')

# Reference data retrieved from HTTP Archive in 06-jan-2014
Config.define('MAX_IMG_REQUESTS_PER_PAGE', 40,
              _('Maximum number of images per page'), 'Image Request Validator')
Config.define('MAX_KB_SINGLE_IMAGE', 26,
              _('Maximum size of a single image'), 'Image Request Validator')
Config.define('MAX_IMG_KB_PER_PAGE', 1028,
              _('Maximum size of images per page'), 'Image Request Validator')
# Reference data retrieved from HTTP Archive in 06-jan-2014
Config.define('MAX_CSS_REQUESTS_PER_PAGE', 8,
              _('Maximum number of external stylesheets per page'), 'CSS Request Validator')
Config.define('MAX_CSS_KB_PER_PAGE_AFTER_GZIP', 46,
              _('Maximum size of stylesheets per page after gzip'), 'CSS Request Validator')

# Reference data retrieved from HTTP Archive in 06-jan-2014
Config.define('MAX_JS_REQUESTS_PER_PAGE', 17,
              _('Maximum number of external scripts per page'), 'JS Request Validator')
Config.define('MAX_JS_KB_PER_PAGE_AFTER_GZIP', 272,
              _('Maximum size of scripts per page after gzip'), 'JS Request Validator')

Config.define('MAX_TITLE_SIZE', 70,
              _('Title tags longer than 70 characters may be truncated in the results'),
              'Title Validator')
Config.define('METATAG_DESCRIPTION_MAX_SIZE', 300,
              _('Description Meta tags longer than 300 characters may be truncated in the results'),
              'Metatag validator')
Config.define('MAX_HEADING_HIEARARCHY_SIZE', 150,
              _('Heading Hierarchy tags longer than 150 characters may be truncated in the results'),
              'Heading Hierarchy Validator')
Config.define('MAX_IMAGE_ALT_SIZE', 70,
              _('Image alt attributes longer than 70 characters may be truncated in the results'),
              'Image Alt Attribute Validator')

Config.define('HTTP_PROXY_HOST', None, _('HTTP Proxy Host to use'), 'Web')
Config.define('HTTP_PROXY_PORT', None, _('HTTP Proxy Port to use'), 'Web')

Config.define('API_PROXY_HOST', None, _('HTTP Proxy Host to use to connect to the API'), 'Web')
Config.define('API_PROXY_PORT', None, _('HTTP Proxy Port to use to connect to the API'), 'Web')

Config.define('COMMIT_ON_REQUEST_END', True, _('Commit on request end'), 'DB')

Config.define('MATERIAL_GIRL_REDISPORT', 7575, _('Redis port'), 'Redis')
Config.define('MATERIAL_GIRL_REDISHOST', 'localhost', _('Redis host'), 'Redis')
Config.define('MATERIAL_GIRL_REDISPASS', None, _('Redis password in case of auth'), 'Redis')
Config.define('MATERIAL_GIRL_LOAD_ON_CACHEMISS', True, _('Load from database on cache miss'), 'Materializer')

Config.define('REQUIRED_META_TAGS', [], _('List of required meta tags'), 'Meta tag Validator')

Config.define('SCHEMA_ORG_ITEMTYPE', [], _('List of Schema.Org ItemType'), 'Schema.Org ItemType')

Config.define('FORCE_CANONICAL', False, _('Force canonical'), 'Force canonical')

Config.define('BLACKLIST_DOMAIN', [], _('Domain blacklist'), 'Domain blacklist')

Config.define('ERROR_HANDLERS', [], _('List of classes to handle errors'), 'General')

Config.define('SEARCH_PROVIDER', 'holmes.search_providers.noexternal.NoExternalSearchProvider', _('Class to handle searching'), 'Models')

Config.define('ELASTIC_SEARCH_PROTOCOL', 'http', _('ElasticSearch protocol (http|https)'), 'ElasticSearchProvider')
Config.define('ELASTIC_SEARCH_HOST', 'localhost', _('ElasticSearch host'), 'ElasticSearchProvider')
Config.define('ELASTIC_SEARCH_PORT', 9200, _('ElasticSearch port'), 'ElasticSearchProvider')
Config.define('ELASTIC_SEARCH_INDEX', 'holmes', _('ElasticSearch index name'), 'ElasticSearchProvider')

Config.define('ELASTIC_SEARCH_MAX_RETRIES', 3, _('ElasticSearch max number of retries'), 'ElasticSearchProvider')

# SENTRY ERROR HANDLER
Config.define('USE_SENTRY', False, _('If set to true errors will be sent to sentry.'), 'Sentry')
Config.define('SENTRY_DSN_URL', '', _('URL to use as sentry DSN.'), 'Sentry')

Config.define('PAGE_COUNT_EXPIRATION_IN_SECONDS', HOUR, _('Expiration for the cache key for each domain page count'), 'Cache')
Config.define('VIOLATION_COUNT_EXPIRATION_IN_SECONDS', HOUR, _('Expiration for the cache key for each domain violation count'), 'Cache')
Config.define('ACTIVE_REVIEW_COUNT_EXPIRATION_IN_SECONDS', HOUR, _('Expiration for the cache key for each domain violation count'), 'Cache')
Config.define('RESPONSE_TIME_AVG_EXPIRATION_IN_SECONDS', HOUR, _('Expiration for the cache key for each domain average response time'), 'Cache')
Config.define('VIOLATIONS_BY_CATEGORY_EXPIRATION_IN_SECONDS', 6 * 60, _('Expiration for the cache key for each domain violation count by category'), 'Cache')
Config.define('TOP_CATEGORY_VIOLATIONS_EXPIRATION_IN_SECONDS', 6 * 60, _('Expiration for the cache key for each domain top violation in a category'), 'Cache')
Config.define('URL_LOCK_EXPIRATION_IN_SECONDS', 30, _('Expiration for the URL lock for each URL'), 'Cache')
Config.define('NEXT_JOB_URL_LOCK_EXPIRATION_IN_SECONDS', 3 * 60, _('Expiration for the URL lock for next jobs'), 'Cache')
Config.define('NEXT_JOBS_COUNT_EXPIRATION_IN_SECONDS', HOUR, _('Expiration for the cache key for next jobs count'), 'Cache')

materials_expiration_in_seconds = {
    'domains_details': 0.5 * MINUTE + 1,
    'violation_count_for_domains': 4 * MINUTE + 11,
    'violation_count_by_category_for_domains': 3 * MINUTE + 11,
    'top_violations_in_category_for_domains': 5 * MINUTE + 17,
    'blacklist_domain_count': 10 * MINUTE + 1,
    'most_common_violations': HOUR + 7,
    'failed_responses_count': HOUR + 13,
}
Config.define('MATERIALS_EXPIRATION_IN_SECONDS', materials_expiration_in_seconds, 'Expire times for materials', 'material')

materials_grace_period_in_seconds = {
    'domains_details': 2 * materials_expiration_in_seconds['domains_details'],
    'violation_count_for_domains': 2 * materials_expiration_in_seconds['violation_count_for_domains'],
    'violation_count_by_category_for_domains': 2 * materials_expiration_in_seconds['violation_count_by_category_for_domains'],
    'top_violations_in_category_for_domains': 2 * materials_expiration_in_seconds['top_violations_in_category_for_domains'],
    'blacklist_domain_count': 2 * materials_expiration_in_seconds['blacklist_domain_count'],
    'most_common_violations': 2 * materials_expiration_in_seconds['most_common_violations'],
    'failed_responses_count': 2 * materials_expiration_in_seconds['failed_responses_count'],
}
Config.define('MATERIALS_GRACE_PERIOD_IN_SECONDS', materials_grace_period_in_seconds, _('Grace period times for materials'), 'material')

materials_lock_timeout_in_seconds = {
    'domains_details': 3 * materials_grace_period_in_seconds['domains_details'] / 4,
    'violation_count_for_domains': 3 * materials_grace_period_in_seconds['violation_count_for_domains'] / 4,
    'violation_count_by_category_for_domains': 3 * materials_grace_period_in_seconds['violation_count_by_category_for_domains'] / 4,
    'top_violations_in_category_for_domains': 3 * materials_grace_period_in_seconds['top_violations_in_category_for_domains'] / 4,
    'blacklist_domain_count': 3 * materials_grace_period_in_seconds['blacklist_domain_count'] / 4,
    'most_common_violations': 3 * materials_grace_period_in_seconds['most_common_violations'] / 4,
    'failed_responses_count': 3 * materials_grace_period_in_seconds['failed_responses_count'] / 4,
}
Config.define('MATERIALS_LOCK_TIMEOUT_IN_SECONDS', materials_lock_timeout_in_seconds, _('Lock timeouts for materials'), 'material')

Config.define('DEFAULT_PAGE_SCORE', 1, _('Page Score for pages that the user includes through the UI'), 'General')
Config.define('PAGE_SCORE_TAX_RATE', 0.1, _('Default tax rate for scoring pages.'), 'General')

Config.define('REQUEST_CACHE_EXPIRATION_IN_SECONDS', HOUR, _('Expiration in seconds for cache storage of responses.'), 'Cache')

Config.define('MAX_URL_LEVELS', 20, _('Maximum levels of URL'))

Config.define('GOOGLE_CLIENT_ID', None, _('Google client ID'))

Config.define('LIMITER_LOCKS_EXPIRATION', 120, _('The expiration for locks in the limiter'))
Config.define('LIMITER_VALUES_CACHE_EXPIRATION', 600, _('The expiration for values in the limiter'))
Config.define('DEFAULT_NUMBER_OF_CONCURRENT_CONNECTIONS', 5, _('Default number of concurrent connections'), 'Limiter')

Config.define('MOST_COMMON_VIOLATIONS_CACHE_EXPIRATION', 3 * HOUR, _('Expiration for the cache key for the most common violations'), 'Cache')
Config.define('MOST_COMMON_VIOLATIONS_SAMPLE_LIMIT', 50000, _('Limit for the size of the Violation sample used in the aggregation'), 'Violation Handler')
Config.define('TOP_CATEGORY_VIOLATIONS_SAMPLE_LIMIT', 50000, _('Size of the sample used in the top violations of a key category for domains'), 'Domain Handler')

throttling_message_type = {
    'new-request': 5,
    'new-page': 2,
    'new-review': 2,
}
Config.define('EVENT_BUS_THROTTLING_MESSAGE_TYPE', throttling_message_type, _('Throttling by message type'), 'Event Bus')

Config.define('SQLALCHEMY_AUTO_FLUSH', True, _('Defines whether auto-flush should be used in sqlalchemy'))

Config.define('DOMAINS_VIOLATIONS_PREFS_EXPIRATION_IN_SECONDS', HOUR, _('Expiration in seconds for domains violations prefs.'), 'Cache')

Config.define('SECRET_KEY', 'set-a-secret-key', _('Secret key to use in JSON Web Token generation'), 'Sessions')
Config.define('SESSION_EXPIRATION', HOUR, _('Time for Session expiration in seconds'), 'Sessions')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class MetaRobotsValidator(Validator):
    META_ROBOTS_NO_INDEX = _('A meta tag with the robots="noindex" '
                             'attribute tells the search engines that '
                             'they should not index this page.')

    META_ROBOTS_NO_FOLLOW = _('A meta tag with the robots="nofollow" '
                              'attribute tells the search engines that they '
                              'should not follow any links in this page.')

    @classmethod
    def get_violation_definitions(cls):
        return {
            'presence.meta.robots.noindex': {
                'title': _('Meta Robots with value of noindex'),
                'description': cls.META_ROBOTS_NO_INDEX,
                'category': _('SEO'),
                'generic_description': cls.META_ROBOTS_NO_INDEX
            },
            'presence.meta.robots.nofollow': {
                'title': _('Meta Robots with value of nofollow'),
                'description': cls.META_ROBOTS_NO_FOLLOW,
                'category': _('SEO'),
                'generic_description': cls.META_ROBOTS_NO_FOLLOW
            }
        }

    def validate(self):
        tags = self.get_meta_tags()

        for tag in tags:
            if tag['key'] == 'robots' and tag['content'] == 'noindex':
                self.add_violation(
                    key='presence.meta.robots.noindex',
                    value=None,
                    points=80
                )

            if tag['key'] == 'robots' and tag['content'] == 'nofollow':
                self.add_violation(
                    key='presence.meta.robots.nofollow',
                    value=None,
                    points=50
                )

    def get_meta_tags(self):
        return self.review.data.get('meta.tags', None)



"""create workers table

Revision ID: 25abb99fc57
Revises: 1a316b08d134
Create Date: 2013-11-13 16:00:05.263655

"""

# revision identifiers, used by Alembic.
revision = '25abb99fc57'
down_revision = '1a316b08d134'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'workers',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('uuid', sa.String(36), nullable=False),
        sa.Column('last_ping', sa.DateTime, nullable=False),
        sa.Column('current_review_id', sa.Integer)
    )

    op.create_foreign_key(
        "fk_worker_current_review", "workers",
        "reviews", ["current_review_id"], ["id"]
    )


def downgrade():
    op.drop_constraint('fk_worker_current_review', 'workers', type_='foreignkey')
    op.drop_table('workers')



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sqlalchemy as sa
from collections import defaultdict

from holmes.models import Base, JsonTypeGzipped
from holmes.models.keys import Key


class Violation(Base):
    __tablename__ = "violations"

    id = sa.Column(sa.Integer, primary_key=True)
    value = sa.Column('value', JsonTypeGzipped, nullable=True)
    points = sa.Column('points', sa.Integer, nullable=False)

    review_id = sa.Column('review_id', sa.Integer, sa.ForeignKey('reviews.id'))
    # review comes from Review relationship
    key_id = sa.Column('key_id', sa.Integer, sa.ForeignKey('keys.id'))

    domain_id = sa.Column('domain_id', sa.Integer, sa.ForeignKey('domains.id'))

    review_is_active = sa.Column('review_is_active', sa.Boolean, default=True, nullable=False)

    def __str__(self):
        return '%s: %s' % (self.key.name, self.value)

    def __repr__(self):
        return str(self)

    def to_dict(self, violation_definitions, _):
        definition = violation_definitions.get(self.key.name, {})

        value = definition.get('value_parser', lambda val: val)(self.value)
        description = _(definition.get('description', '%s'))

        if value:
            try:
                description = (description % value)
            except TypeError:
                pass

        return {
            'key': self.key.name,
            'title': _(definition.get('title', _('undefined'))),
            'description': description,
            'points': self.points,
            'category': _(definition.get('category', _('undefined')))
        }

    @classmethod
    def get_most_common_violations_names(cls, db, sample_limit=50000):
        sample = db \
            .query(
                Violation.id,
                Violation.key_id,
                Key.name.label('key_name')
            ) \
            .filter(Violation.key_id == Key.id) \
            .filter(Violation.review_is_active == True) \
            .order_by(Violation.id.desc()) \
            .limit(sample_limit) \
            .subquery()

        return db \
            .query(
                sample.columns.key_name,
                sa.func.count(sample.columns.key_id).label('count')
            ) \
            .group_by(sample.columns.key_id) \
            .order_by('count desc').all()

    @classmethod
    def get_group_by_key_id_for_all_domains(cls, db):

        from holmes.models.violation import Violation  # to avoid circular dependency
        from holmes.models.domain import Domain  # to avoid circular dependency

        return db \
            .query(
                Violation.key_id,
                Domain.name.label('domain_name'),
                sa.func.count(Violation.id).label('violation_count')
            ) \
            .filter(Domain.id == Violation.domain_id) \
            .filter(Violation.review_is_active == True) \
            .group_by(Violation.key_id, Domain.id) \
            .order_by('violation_count DESC') \
            .all()

    @classmethod
    def get_group_by_category_id_for_all_domains(cls, db):
        from holmes.models.keys import Key  # to avoid circular dependency
        from holmes.models.violation import Violation  # to avoid circular dependency

        data = db \
            .query(
                Violation.domain_id,
                Key.name,
                Key.category_id,
                sa.func.count(Key.category_id).label('violation_count')
            ) \
            .filter(Key.id == Violation.key_id) \
            .filter(Violation.review_is_active == True) \
            .group_by(Violation.domain_id) \
            .group_by(Key.category_id) \
            .order_by('violation_count DESC') \
            .all()

        result = defaultdict(list)

        for item in data:
            result[item.domain_id].append({
                'key_name': item.name,
                'category_id': item.category_id,
                'violation_count': item.violation_count
            })

        return result

    @classmethod
    def get_group_by_value_for_key(cls, db, key_name):
        from holmes.models.keys import Key  # to avoid circular dependency
        from holmes.models.violation import Violation  # to avoid circular dependency

        return db \
            .query(
                Violation.value,
                sa.func.count(Violation.key_id).label('count')
            ) \
            .filter(Key.name == key_name) \
            .filter(Key.id == Violation.key_id) \
            .filter(Violation.review_is_active == True) \
            .group_by(Violation.value) \
            .order_by('count DESC') \
            .all()

    @classmethod
    def get_top_in_category_for_all_domains(cls, db, limit=1000):
        from holmes.models.keys import Key  # to avoid circular dependency
        from holmes.models.domain import Domain  # to avoid circular dependency
        from holmes.models.violation import Violation  # to avoid circular dependency

        return db \
            .query(
                Domain.name,
                Key.category_id,
                Key.name,
                sa.func.count(Key.category_id).label('violation_count')
            ) \
            .filter(Domain.id == Violation.domain_id) \
            .filter(Violation.key_id == Key.id) \
            .filter(Violation.review_is_active == True) \
            .group_by(Domain.id, Key.category_id, Key.id) \
            .order_by('violation_count DESC') \
            .limit(limit) \
            .all()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging

from holmes.facters import Facter
from holmes.utils import _


class JSFacter(Facter):
    @classmethod
    def get_fact_definitions(cls):
        return {
            'page.js': {
                'title': _('JS'),
                'description': lambda value: list(value),
                'unit': 'js',
                'category': _('Static'),
            },
            'total.requests.js': {
                'title': _('Total JS requests'),
                'description': lambda value: value,
                'category': _('HTTP'),
                'unit': 'number'
            },
            'total.size.js': {
                'title': _('Total JS size'),
                'description': lambda value: '%d' % value,
                'unit': 'kb',
                'category': _('SEO'),
            },
            'total.size.js.gzipped': {
                'title': _('Total JS size gzipped'),
                'description': lambda value: '%d' % value,
                'unit': 'kb',
                'category': _('SEO')
            }
        }

    def get_facts(self):
        js_files = self.get_js_requests()

        self.review.data['page.js'] = set()
        self.review.data['total.size.js'] = 0
        self.review.data['total.size.js.gzipped'] = 0

        self.add_fact(
            key='page.js',
            value=set(),
        )

        self.add_fact(
            key='total.size.js',
            value=0,
        )

        self.add_fact(
            key='total.size.js.gzipped',
            value=0,
        )

        num_js = 0

        js_to_get = set()

        for js_file in js_files:
            src = js_file.get('src')

            src = self.normalize_url(src)
            if src:
                js_to_get.add(src)
                num_js += 1

        self.add_fact(
            key='total.requests.js',
            value=num_js,
        )

        for url in js_to_get:
            self.async_get(url, self.handle_url_loaded)

    def handle_url_loaded(self, url, response):
        logging.debug('Got response (%s) from %s!' % (response.status_code,
                                                      url))

        self.review.facts['page.js']['value'].add(url)
        self.review.data['page.js'].add((url, response))

        if response.text:
            size_js = len(response.text) / 1024.0
            size_gzip = len(self.to_gzip(response.text)) / 1024.0
        else:
            size_js = 0
            size_gzip = 0

        self.review.facts['total.size.js']['value'] += size_js
        self.review.data['total.size.js'] += size_js

        self.review.facts['total.size.js.gzipped']['value'] += size_gzip
        self.review.data['total.size.js.gzipped'] += size_gzip

    def get_js_requests(self):
        return self.reviewer.current_html.cssselect('script[src]')



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sqlalchemy as sa

from holmes.models import Base


class UsersViolationsPrefs(Base):
    __tablename__ = "users_violations_prefs"

    id = sa.Column(sa.Integer, primary_key=True)
    user_id = sa.Column('user_id', sa.Integer, sa.ForeignKey('users.id'))
    key_id = sa.Column('key_id', sa.Integer, sa.ForeignKey('keys.id'))
    is_active = sa.Column('is_active', sa.Boolean, default=True, nullable=False)

    def __str__(self):
        return '%s: %s' % (self.user.email, self.key.name)

    def __repr__(self):
        return str(self)

    def to_dict(self):
        return {
            'user': self.user.email,
            'key': self.key.name,
            'is_active': self.is_active
        }

    @classmethod
    def get_prefs(cls, db, user):
        prefs = db \
            .query(UsersViolationsPrefs) \
            .filter(UsersViolationsPrefs.user_id == user.id) \
            .all()

        result = {}
        for pref in prefs:
            result.update({pref.key.name: pref.is_active})
        return result

    @classmethod
    def update_by_user(cls, db, user, data):
        from holmes.models import Key

        if not user or not data:
            return

        for item in data:
            if 'key' not in item or 'is_active' not in item:
                continue

            key = Key.get_by_name(db, item.get('key'))
            if not key:
                continue

            is_active = item.get('is_active', True)

            db \
                .query(UsersViolationsPrefs) \
                .filter(
                    UsersViolationsPrefs.user_id == user.id,
                    UsersViolationsPrefs.key_id == key.id
                ) \
                .update(
                    {'is_active': is_active}
                )

        db.flush()

    @classmethod
    def delete_prefs(cls, db, user, items):
        from holmes.models import Key

        for key_name in items:
            key = Key.get_by_name(db, key_name)
            if not key:
                continue

            db \
                .query(UsersViolationsPrefs) \
                .filter(
                    UsersViolationsPrefs.key_id == key.id,
                    UsersViolationsPrefs.user_id == user.id
                ) \
                .delete()

        db.flush()

    @classmethod
    def insert_pref(cls, db, user, key, is_active):
        pref = UsersViolationsPrefs(user=user, key=key, is_active=is_active)
        db.add(pref)
        db.flush()

    @classmethod
    def insert_prefs(cls, db, user, items):
        from holmes.models import Key

        if not user:
            return

        data = []
        for key_name in items:
            key = Key.get_by_name(db, key_name)

            if not key:
                continue

            data.append({
                'user_id': user.id,
                'key_id': key.id,
                'is_active': True
            })

        db.execute(UsersViolationsPrefs.__table__.insert(), data)



"""Pages, Domain and Review indexes for getting by uuid

Revision ID: 4ab7a2e91f2d
Revises: 5edcaddd6c9
Create Date: 2013-11-27 15:36:05.865518

"""

# revision identifiers, used by Alembic.
revision = '4ab7a2e91f2d'
down_revision = '5edcaddd6c9'

from alembic import op


def upgrade():
    op.create_index('idx_pages_uuid', 'pages', ['uuid'])
    op.create_index('idx_reviews_uuid', 'reviews', ['uuid'])


def downgrade():
    op.drop_index('idx_pages_uuid', 'pages')
    op.drop_index('idx_reviews_uuid', 'reviews')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from collections import defaultdict
import sqlalchemy as sa
from sqlalchemy import func, distinct
from datetime import date, timedelta, datetime

from ujson import dumps

from holmes.utils import get_status_code_title
from holmes.models import Base


class Request(Base):
    __tablename__ = "requests"

    id = sa.Column(sa.Integer, primary_key=True)
    domain_name = sa.Column('domain_name', sa.String(120), nullable=False)
    url = sa.Column('url', sa.Text(), nullable=False)
    effective_url = sa.Column('effective_url', sa.Text(), nullable=False)
    status_code = sa.Column('status_code', sa.Integer, nullable=False)
    response_time = sa.Column('response_time', sa.Float, nullable=False)
    completed_date = sa.Column('completed_date', sa.Date, nullable=False)
    review_url = sa.Column('review_url', sa.Text(), nullable=False)

    def to_dict(self):
        return {
            'domain_name': str(self.domain_name),
            'url': self.url,
            'effective_url': self.effective_url,
            'status_code': self.status_code,
            'response_time': self.response_time,
            'completed_date': self.completed_date,
            'review_url': self.review_url
        }

    def __str__(self):
        return "%s (%s)" % (self.url, self.status_code)

    def __repr__(self):
        return str(self)

    @classmethod
    def get_status_code_info(self, domain_name, db):
        result = []

        query = db \
            .query(
                Request.status_code,
                sa.func.count(Request.status_code).label('total')
            ) \
            .filter(Request.domain_name == domain_name) \
            .group_by(Request.status_code) \
            .all()

        for i in query:
            result.append({
                'code': i.status_code,
                'title': get_status_code_title(i.status_code),
                'total': i.total
            })

        return result

    @classmethod
    def get_requests_by_status_code(self, domain_name, status_code, db, current_page=1, page_size=10):
        lower_bound = (current_page - 1) * page_size
        upper_bound = lower_bound + page_size

        requests = db \
            .query(Request.id, Request.url, Request.review_url, Request.completed_date) \
            .filter(Request.domain_name == domain_name) \
            .filter(Request.status_code == status_code) \
            .order_by('completed_date desc')[lower_bound:upper_bound]

        return requests

    @classmethod
    def get_requests_by_status_count(self, domain_name, status_code, db):
        return db \
            .query(func.count(Request.id)) \
            .filter(Request.domain_name == domain_name) \
            .filter(Request.status_code == status_code) \
            .scalar()

    @classmethod
    def get_last_requests(self, db, current_page=1, page_size=10,
                          domain_filter=None, status_code_filter=None):
        lower_bound = (current_page - 1) * page_size
        upper_bound = lower_bound + page_size

        query = db.query(Request)

        if domain_filter:
            from holmes.models.domain import Domain
            domain = Domain.get_domain_by_name(domain_filter, db)
            if domain:
                query = query.filter(Request.domain_name == domain.name)

        if status_code_filter and int(status_code_filter):
            query = query.filter(Request.status_code == status_code_filter)

        return query.order_by('id desc')[lower_bound:upper_bound]

    @classmethod
    def get_requests_count_by_status(self, db, limit=1000):
        per_domains = {'_all': defaultdict(int)}

        from holmes.models.domain import Domain
        for domain in db.query(Domain).all():
            requests = db \
                .query(
                    Request.status_code,
                    sa.func.count(Request.id).label('count')
                ) \
                .filter(Request.domain_name == domain.name) \
                .group_by(Request.status_code) \
                .order_by('count DESC') \
                .limit(limit) \
                .all()
            per_domains[domain.name] = requests

            # calculating all domains by counting each domain
            for req in requests:
                per_domains['_all'][req[0]] += req[1]

        per_domains['_all'] = per_domains['_all'].items()

        return per_domains

    @classmethod
    def delete_old_requests(self, db, config, limit=1000):
        dt = date.today() - timedelta(days=config.DAYS_TO_KEEP_REQUESTS)

        older_requests = db \
            .query(Request) \
            .filter(Request.completed_date <= dt) \
            .limit(limit) \

        older_requests_ids = [item.id for item in older_requests]

        if not older_requests_ids:
            return None

        return db \
            .query(Request) \
            .filter(Request.id.in_(older_requests_ids)) \
            .delete(synchronize_session=False)

    @classmethod
    def get_all_status_code(self, db):
        status_code_list = db \
            .query(distinct(Request.status_code).label('status_code')) \
            .all()

        result = []

        for value in status_code_list:
            result.append({
                'statusCode': value.status_code,
                'statusCodeTitle': get_status_code_title(value.status_code)
          })

        return result

    @classmethod
    def save_requests(cls, db, publish, page, requests):
        if not requests:
            return

        domain_name = page.domain.name
        page_url = page.url

        data = []
        for url, response in requests:
            request_time = response.request_time
            effective_url = response.effective_url
            status_code = response.status_code

            data.append({
                'domain_name': domain_name,
                'url': url,
                'effective_url': effective_url,
                'status_code': int(status_code),
                'response_time': request_time,
                'completed_date': datetime.now().date(),
                'review_url': page_url
            })

        db.execute(Request.__table__.insert(), data)

        url = url.encode('utf-8')

        publish(dumps({
            'type': 'new-request',
            'url': str(url)
        }))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from ujson import loads
from tornado.gen import coroutine

from holmes.handlers import BaseHandler
from holmes.models import User


class UserLocaleHandler(BaseHandler):

    @coroutine
    def get(self):
        user = self.get_authenticated_user()

        if not user:
            return

        self.write_json({'locale': user.locale})
        self.finish()

    @coroutine
    def post(self):
        user = self.get_authenticated_user()

        if not user:
            return

        post_data = loads(self.request.body)
        locale = post_data.get('locale', None)

        if locale:
            User.update_locale(self.db, user, locale)

        self.write_json(self._('OK'))
        self.finish()



"""Requests: Adding index for completed_date

Revision ID: 2db70040b01b
Revises: 519b66176c0c
Create Date: 2014-04-28 15:58:12.331243

"""

# revision identifiers, used by Alembic.
revision = '2db70040b01b'
down_revision = '519b66176c0c'

from alembic import op


def upgrade():
    op.create_index('idx_completed_date', 'requests', ['completed_date'])


def downgrade():
    op.drop_index('idx_completed_date', 'requests')



"""drop workers table

Revision ID: 519b66176c0c
Revises: 379d301c24ed
Create Date: 2014-04-10 15:53:05.866376

"""

# revision identifiers, used by Alembic.
revision = '519b66176c0c'
down_revision = '379d301c24ed'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.drop_constraint('uk_uuid', 'workers', type_='unique')
    op.drop_index('idx_last_ping', 'workers')
    op.drop_table('workers')


def downgrade():
    op.create_table(
        'workers',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('uuid', sa.String(36), nullable=False),
        sa.Column('last_ping', sa.DateTime, nullable=False),
        sa.Column('current_url', sa.Text, nullable=True)
    )

    op.create_unique_constraint('uk_uuid', 'workers', ['uuid'])
    op.create_index('idx_last_ping', 'workers', ['last_ping'])



#!/usr/bin/python
# -*- coding: utf-8 -*-

import re

from holmes.validators.base import Validator
from holmes.utils import _

URL_RE = re.compile(
    r'^(?:http|ftp)s?://'  # http:// or https://
    r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
    r'localhost|'  # localhost...
    r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
    r'(?::\d+)?'  # optional port
    r'(?P<relative>(?:/?|[/?]\S+))$', re.IGNORECASE)

HTML_ENTITIES = re.compile(r'(&amp;|&apos;|&quot;|&gt;|&lt;)')
INVALID_CHARS = re.compile(r'(&|\'|"|>|<)')


class SitemapValidator(Validator):
    MAX_SITEMAP_SIZE = 10  # 10 MB
    MAX_LINKS_SITEMAP = 50000

    @classmethod
    def get_violation_definitions(cls):
        return {
            'sitemap.not_found': {
                'title': _('SiteMap file not found'),
                'description': _("The sitemap file was not found at '%s'."),
                'category': _('SEO'),
                'generic_description': _(
                    'Validates the absent of a SiteMap file on the page.')
            },
            'sitemap.empty': {
                'title': _('SiteMap file is empty'),
                'description': _(
                    "The sitemap file was found at '%s', but was empty."),
                'category': _('Semantics'),
                'generic_description': _(
                    'Validates the emptiness of site\'s SiteMap file.')
            },
            'total.size.sitemap': {
                'title': _('SiteMap file is too big'),
                'description': _(
                    "The sitemap file was found at '%(url)s', but was "
                    "too big (%(size).2fMB)."),
                'category': _('Performance'),
                'generic_description': _(
                    'Validates the size of site\'s SiteMap file.')
            },
            'total.links.sitemap': {
                'title': _('SiteMap file contains too many links'),
                'description': _(
                    "The sitemap file was found at '%(url)s', but contained "
                    "too many links (%(links)d). Maybe splitting it would "
                    "help?"),
                'category': _('SEO'),
                'generic_description': _(
                    'Validates the number of links containing in the site\'s '
                    'SiteMap file. This limit is configurable on Holmes '
                    'configuration.'
                )
            },
            'sitemap.links.not_encoded': {
                'title': _('SiteMap file contains unencoded links'),
                'description': _(
                    "The sitemap file was found at '%(url)s', but contains "
                    "unencoded links (%(links)s)."),
                'category': _('Semantics'),
                'generic_description': _(
                    'Validates the value of site\'s SiteMap file links.'
                )
            }
        }

    def validate(self):
        if not self.reviewer.is_root():
            return

        for sitemap, size in self.review.data['sitemap.files.size'].items():
            response = self.review.data['sitemap.data'][sitemap]

            if response.status_code > 399:
                self.add_violation(
                    key='sitemap.not_found',
                    value=sitemap,
                    points=100
                )
                return

            if not response.text.strip():
                self.add_violation(
                    key='sitemap.empty',
                    value=sitemap,
                    points=100
                )
                return

            size_mb = (size / 1024.0)
            urls_count = self.review.data['sitemap.files.urls'][sitemap]
            not_encoded_links = 0

            if size_mb > self.MAX_SITEMAP_SIZE:
                self.add_violation(
                    key='total.size.sitemap',
                    value={
                        'url': sitemap,
                        'size': size_mb
                    },
                    points=10
                )

            if urls_count > self.MAX_LINKS_SITEMAP:
                self.add_violation(
                    key='total.links.sitemap',
                    value={
                        'url': sitemap,
                        'links': urls_count
                    },
                    points=10
                )

            for url in self.review.data['sitemap.urls'][sitemap]:
                match = URL_RE.match(url)

                if not match:
                    continue

                parse = match.groupdict()
                relative = parse['relative']
                encoded = True

                try:
                    str(relative).encode('utf-8')
                except (UnicodeEncodeError, UnicodeDecodeError):
                    encoded = False

                relative = HTML_ENTITIES.sub('', relative)
                encoded = encoded and not INVALID_CHARS.findall(relative)

                if not encoded:
                    not_encoded_links += 1

                self.send_url(url, self.reviewer.page_score / float(urls_count), response)

            if not_encoded_links > 0:
                self.add_violation(
                    key='sitemap.links.not_encoded',
                    value={
                        'url': sitemap,
                        'links': not_encoded_links
                    },
                    points=10
                )

        self.flush()



"""Add locale field in User table

Revision ID: 5592365e4624
Revises: 47fac4be8ca7
Create Date: 2014-07-15 16:54:18.394370

"""

# revision identifiers, used by Alembic.
revision = '5592365e4624'
down_revision = '47fac4be8ca7'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        'users',
        sa.Column('locale', type_=sa.String(10))
    )


def downgrade():
    op.drop_column('users', 'locale')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class BodyValidator(Validator):
    @classmethod
    def get_violation_definitions(cls):
        return {
            'page.body.not_found': {
                'title': _('Page body not found'),
                'description': _('Body was not found on %s.'),
                'category': _('Semantics'),
                'generic_description': _(
                    'The <body> tag defines the documents body\'s, it\'s contains '
                    'all the content of an HTML document. The ausence of a this tag '
                    'represents a no content and sematically invalid webpage.'
                ),
            }
        }

    def validate(self):
        body = self.get_body()

        if not body:

            page_url = self.reviewer.page_url

            self.add_violation(
                key='page.body.not_found',
                value=page_url,
                points=50
            )

    def get_body(self):
        return self.review.data.get('page.body', None)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from os.path import join

# Deferred imports
try:
    from urllib.parse import urlparse  # Py3
except ImportError:
    from urlparse import urlparse

import inspect
import email.utils as eut
from datetime import datetime

import codecs
from box.util.rotunicode import RotUnicode
codecs.register(RotUnicode.search_function)

import lxml.html
import logging

from holmes.config import Config
from holmes.facters import Facter
from holmes.validators.base import Validator
from holmes.models import Page
from holmes.utils import get_domain_from_url


class InvalidReviewError(RuntimeError):
    pass


class ReviewDAO(object):
    def __init__(self, page_uuid, page_url, last_modified=None, expires=None):
        self.page_uuid = page_uuid
        self.page_url = page_url
        self.last_modified = last_modified
        self.expires = expires
        self.facts = {}
        self.violations = []
        self.data = {}
        self._current = None
        self.requests = []

    def add_fact(self, key, value):
        self.facts[key] = {
            'key': key,
            'value': value
        }

    def add_violation(self, key, value, points):
        self.violations.append({
            'key': key,
            'value': value,
            'points': points
        })

    def to_dict(self):
        return {
            'page_uuid': self.page_uuid,
            'page_url': self.page_url,
            'facts': self.facts.values(),
            'violations': self.violations,
            'lastModified': self.last_modified,
            'expires': self.expires,
            'requests': self.requests
        }


class Reviewer(object):
    def __init__(
            self, api_url, page_uuid, page_url, page_score,
            config=None, validators=[], facters=[], search_provider=None, async_get=None,
            wait=None, wait_timeout=None, db=None, cache=None, publish=None,
            fact_definitions=None, violation_definitions=None, girl=None):

        self.db = db
        self.cache = cache
        self.girl = girl
        self.publish = publish

        self.api_url = api_url

        self.page_uuid = page_uuid
        self.page_url = page_url
        self.page_score = page_score

        self.domain_name, self.domain_url = get_domain_from_url(self.page_url)

        self.ping_method = None

        self.review_dao = ReviewDAO(self.page_uuid, self.page_url)

        assert isinstance(config, Config), 'config argument must be an instance of holmes.config.Config'
        self.config = config

        for facter in facters:
            message = 'All facters must subclass holmes.facters.Facter (Error: %s)' % facter.__class__.__name__
            assert inspect.isclass(facter), message
            assert issubclass(facter, Facter), message

        for validator in validators:
            message = 'All validators must subclass holmes.validators.base.Validator (Error: %s)' % validator.__class__.__name__
            assert inspect.isclass(validator), message
            assert issubclass(validator, Validator), message

        self.validators = validators
        self.facters = facters

        self.search_provider = search_provider

        self.responses = {}
        self.raw_responses = {}
        self.status_codes = {}

        self.async_get_func = async_get
        self._wait_for_async_requests = wait
        self._wait_timeout = wait_timeout

        self.fact_definitions = fact_definitions
        self.violation_definitions = violation_definitions

    def get_domains_violations_prefs_by_key(self, key_name):
        if key_name is None:
            return None

        if self.violation_definitions is None:
            return None

        default_value = None

        key = self.violation_definitions.get(key_name, None)
        if key is not None:
            default_value = key.get('default_value')

        domains_prefs = self.cache.get_domain_violations_prefs(self.domain_name)

        if domains_prefs:
            item = next((item for item in domains_prefs if item['key'] == key_name), None)
            if item:
                return item['value']

        return default_value

    def ping(self):
        if self.ping_method is not None:
            self.ping_method()

    def _async_get(self, url, handler, method='GET', **kw):
        if self.async_get_func:
            self.async_get_func(url, self.handle_async_get(handler), method, **kw)

    def handle_async_get(self, handler):
        def handle(url, response):
            if not hasattr(response, 'from_cache') or not response.from_cache:
                response.from_cache = False
                self.review_dao.requests.append((url, response))

            handler(url, response)

        return handle

    def review(self):
        self.load_content(self.content_loaded)
        self.wait_for_async_requests()

    def load_content(self, callback):
        self._async_get(self.page_url, callback)

    def content_loaded(self, url, response):
        if response.status_code > 399 or response.text is None:
            if response.text:
                headers = None
                if response.status_code == 404:
                    text = None
                else:
                    text = response.text.decode('rotunicode')
            else:
                text = 'Empty response.text'
                headers = response.headers

            msg = "Could not load '%s' (%s) - %s!" % (url,
                                                      response.status_code,
                                                      text)
            logging.error(msg)
            if headers is not None:
                logging.warning('Response is from cache: %s' % response.from_cache)
                logging.warning('Headers for "%s": %s' % (url, headers))
            return

        logging.debug('Content for url %s loaded.' % url)

        last_modified = None

        modified = response.headers.get('Last-Modified', None)
        if modified is not None:
            date = eut.parsedate(modified)
            if date is not None:
                last_modified = datetime(*date[:6])

        self.review_dao.last_modified = last_modified

        expires = None

        expires_key = response.headers.get('Expires', None)
        if expires_key:
            date = eut.parsedate(expires_key)
            if date is not None:
                expires = datetime(*date[:6])

        self.review_dao.expires = expires

        self._current = response

        try:
            self._current.html = lxml.html.fromstring(response.text)
        except (lxml.etree.XMLSyntaxError, lxml.etree.ParserError):
            self._current.html = None

        self.run_facters()
        self.wait_for_async_requests()

        self.run_validators()
        self.wait_for_async_requests()

        self.save_review()

    @property
    def current(self):
        return self._current

    @property
    def current_html(self):
        if not hasattr(self.current, 'html') or self.current.html is None:
            return lxml.html.HtmlElement()
        else:
            return self.current.html

    def run_facters(self):
        for facter in self.facters:
            self.ping()
            logging.debug('---------- Started running facter %s ---------' % facter.__name__)
            facter_instance = facter(self)
            facter_instance.get_facts()

    def run_validators(self):
        for validator in self.validators:
            self.ping()
            logging.debug('---------- Started running validator %s ---------' % validator.__name__)
            validator_instance = validator(self)
            validator_instance.validate()

    def get_url(self, url):
        return join(self.api_url.rstrip('/'), url.lstrip('/'))

    def enqueue(self, urls):
        if not urls:
            return

        for url, score in urls:
            Page.add_page(
                self.db,
                self.cache,
                url,
                score,
                self.async_get_func,
                self.publish,
                self.config,
                self.girl,
                self.violation_definitions,
                self.handle_page_added
            )

        self.wait_for_async_requests()

    def handle_page_added(self, (url, result, page)):
        if not result:
            error_message = "Could not enqueue page '" + url + "'! Error: %s"
            logging.error(error_message % page)

        self.ping()

    def add_fact(self, key, value):
        self.review_dao.add_fact(key, value)

    def add_violation(self, key, value, points):
        self.review_dao.add_violation(key, value, points)

    def save_review(self):
        from holmes.models import Review

        data = self.review_dao.to_dict()

        Review.save_review(
            self.page_uuid, data, self.db, self.search_provider,
            self.fact_definitions, self.violation_definitions,
            self.cache, self.publish, self.config
        )

    def wait_for_async_requests(self):
        self._wait_for_async_requests(self._wait_timeout)

    def is_root(self):
        result = urlparse(self.page_url)
        return '{0}://{1}'.format(result.scheme, result.netloc) == self.page_url.rstrip('/')



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
from os.path import abspath, join, dirname
import logging
import gettext

import jwt
from tornado import httputil
from six.moves.urllib.parse import urlparse


EMPTY_DOMAIN_RESULT = ('', '')

locale_path = abspath(join(dirname(__file__), 'i18n/locale/'))
languages = {}

def _(message):
    return message

def load_languages():
    global languages

    for language in os.listdir(locale_path):
        languages[language] = gettext.translation('api', locale_path, languages=[language])

def install_i18n(language="en_US"):
    global languages

    english = languages.get("en_US")
    lang = languages.get(language, english)
    return lang.ugettext


def get_domain_from_url(url, default_scheme='http'):
    if not url:
        return EMPTY_DOMAIN_RESULT

    scheme = default_scheme
    result = urlparse(url)
    domain = result.netloc

    if (result.scheme and result.scheme in ['http', 'https']):
        scheme = result.scheme

    if (result.scheme and result.scheme not in ['http', 'https']) and not domain:
        domain = result.scheme
    else:
        if not domain:
            domain = result.path.split('/')[0]
            if not '.' in domain:
                return EMPTY_DOMAIN_RESULT

        if ':' in domain:
            domain = domain.split(':')[0]

    original_domain = domain.strip()
    domain = original_domain.replace('www.', '')

    return domain, '%s://%s' % (scheme, original_domain)


def get_class(klass):
    module_name, class_name = klass.rsplit('.', 1)

    module = __import__(module_name)

    if '.' in module_name:
        module = reduce(getattr, module_name.split('.')[1:], module)

    return getattr(module, class_name)


def load_classes(classes=None, classes_to_load=None, default=None):
    if classes_to_load is None:
        classes_to_load = default

    if classes is None:
        classes = []

    for class_full_name in classes_to_load:
        if isinstance(class_full_name, (tuple, set, list)):
            load_classes(classes, class_full_name)
            continue

        try:
            klass = get_class(class_full_name)
            classes.append(klass)
        except ValueError:
            logging.warn('Invalid class name [%s]. Will be ignored.' % class_full_name)
        except AttributeError:
            logging.warn('Class [%s] not found. Will be ignored.' % class_full_name)
        except ImportError:
            logging.warn('Module [%s] not found. Will be ignored.' % class_full_name)

    return classes


def get_status_code_title(status_code):
    status_code = int(status_code)
    try:
        title = httputil.responses[status_code]
    except KeyError:
        if status_code == 599:
            title = 'Tornado Timeout'
        else:
            title = 'Unknown'

    return title


def is_valid(url):
    try:
        return urlparse(url)
    except ValueError:
        return None


def count_url_levels(url):
    parse = is_valid(url)
    if parse:
        path = parse.path
        if path.startswith('/'):
            path = path[1:]
        return len(path.split('/'))
    return None


class Jwt(object):
    '''Json Web Tokens encoding/decoding utility class.
    Usage:
    >>> jwt = Jwt('SECRET')
    >>> token = jwt.encode(dict(sub='user@email.com', iss='provider',
                           token='123456789', iat=now(),
                           exp=datetime_expiration))
    >>> jwt.decode(token)
    {'sub':'user@email.com', 'iss':'provider', 'token':'123456789',
     'iat': <datetime>, 'exp': <datetime>}
    >>> jwt.try_to_decode('invalid-token')
    (False, None)
    '''

    def __init__(self, secret, algo='HS512'):
        self.secret = secret
        self.algo = algo

    def encode(self, payload):
        '''Encodes the payload returning a Json Web Token
        '''
        return jwt.encode(payload, self.secret, self.algo)

    def decode(self, encrypted_payload):
        '''Decodes the Json Web Token returning the payload
        '''
        return jwt.decode(encrypted_payload, self.secret)

    def try_to_decode(self, encrypted_payload):
        '''Tries to decrypt the given encrypted and returns a tuple with
        a decrypted boolean flag and the decrypted object if success is True
        '''
        try:
            return True, self.decode(encrypted_payload)
        except (jwt.ExpiredSignature, jwt.DecodeError, AttributeError):
            return False, None



"""Create some requests indexes

Revision ID: d8f500d9168
Revises: 234e746c4c28
Create Date: 2014-01-30 15:00:25.371123

"""

# revision identifiers, used by Alembic.
revision = 'd8f500d9168'
down_revision = '234e746c4c28'

from alembic import op


def upgrade():
    op.create_index(
        'idx_domain_status_code',
        'requests',
        ['domain_name', 'status_code']
    )


def downgrade():
    op.drop_index('idx_domain_status_code', 'requests')



"""Added index for name in keys_category

Revision ID: 2901a4935e21
Revises: 2db70040b01b
Create Date: 2014-05-12 19:16:34.562972

"""

# revision identifiers, used by Alembic.
revision = '2901a4935e21'
down_revision = '2db70040b01b'

from alembic import op


def upgrade():
    op.create_unique_constraint('uk_name', 'keys_category', ['name'])


def downgrade():
    op.drop_constraint('uk_name', 'keys_category', type_='unique')



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sqlalchemy as sa
from collections import defaultdict

from holmes.models import Base, JsonType


class DomainsViolationsPrefs(Base):
    __tablename__ = "domains_violations_prefs"

    id = sa.Column(sa.Integer, primary_key=True)
    domain_id = sa.Column('domain_id', sa.Integer, sa.ForeignKey('domains.id'))
    key_id = sa.Column('key_id', sa.Integer, sa.ForeignKey('keys.id'))
    value = sa.Column('value', JsonType, nullable=True)

    def __str__(self):
        return '%s (%s): %s' % (self.key.name, self.domain.name, self.value)

    def __repr__(self):
        return str(self)

    def to_dict(self):
        return {
            'domain': self.domain.name,
            'key': self.key.name,
            'value': self.value
        }

    @classmethod
    def get_all_domains_violations_prefs(cls, db):
        return db.query(DomainsViolationsPrefs).all()

    @classmethod
    def get_domains_violations_prefs(cls, db):
        data = cls.get_all_domains_violations_prefs(db)

        prefs = defaultdict(dict)

        for d in data:
            prefs[d.domain.name].update({d.key.name: d.value})

        return prefs

    @classmethod
    def get_domains_violations_prefs_by_domain(cls, db, domain_name):
        from holmes.models import Domain

        prefs = db \
            .query(DomainsViolationsPrefs) \
            .filter(
                Domain.id == DomainsViolationsPrefs.domain_id,
                Domain.name == domain_name
            ) \
            .all()

        result = []
        for pref in prefs:
            result.append({'key': pref.key.name, 'value': pref.value})
        return result

    @classmethod
    def insert_domains_violations_prefs(cls, db, domain, key, value):
        pref = DomainsViolationsPrefs(domain=domain, key=key, value=value)
        db.add(pref)
        db.flush()
        # FIXME
        db.commit()

    @classmethod
    def insert_default_violations_values_for_domain(cls, db, domain, keys, violation_definitions, cache):
        for key_name in keys:
            key = violation_definitions.get(key_name)['key']
            value = violation_definitions.get(key_name).get('default_value', None)
            DomainsViolationsPrefs.insert_domains_violations_prefs(
                db, domain, key, value
            )

        cache.delete_domain_violations_prefs(domain.name)

    @classmethod
    def insert_default_violations_values_for_all_domains(
        cls, db, default_violations_values, violation_definitions, cache):

        from holmes.models import Domain

        domains_violations_prefs = DomainsViolationsPrefs.get_domains_violations_prefs(db)

        domains = Domain.get_all_domains(db)

        for domain in domains:
            domain_data = domains_violations_prefs.get(domain.name, None)

            if domain_data:
                keys = set(default_violations_values.keys()) - set(domain_data.keys())
            else:
                keys = default_violations_values.keys()

            DomainsViolationsPrefs.insert_default_violations_values_for_domain(
                db,
                domain,
                keys,
                violation_definitions,
                cache
            )

    @classmethod
    def update_by_domain(cls, db, cache, domain, data):
        from holmes.models import Key

        if not domain or not data:
            return

        for item in data:
            if 'key' not in item or 'value' not in item:
                continue

            db \
                .query(DomainsViolationsPrefs) \
                .filter(
                    DomainsViolationsPrefs.domain_id == domain.id,
                    Key.name == item.get('key'),
                    Key.id == DomainsViolationsPrefs.key_id
                ) \
                .update(
                    {'value': item.get('value')},
                    synchronize_session='fetch'
                )

        db.flush()

        cache.delete_domain_violations_prefs(domain.name)



"""Change violation to contain less info

Revision ID: 21087e990aa8
Revises: 256faac818c5
Create Date: 2013-12-02 22:11:34.036524

"""

# revision identifiers, used by Alembic.
revision = '21087e990aa8'
down_revision = '256faac818c5'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.drop_column('violations', 'title')
    op.drop_column('violations', 'description')
    op.add_column('violations', sa.Column('value', sa.Text, nullable=True))


def downgrade():
    op.add_column('violations', sa.Column('title', sa.String(2000), nullable=False))
    op.add_column('violations', sa.Column('description', sa.Text, nullable=False))
    op.drop_column('violations', 'value')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class CSSRequestsValidator(Validator):
    @classmethod
    def get_violation_definitions(cls):
        return {
            'total.requests.css': {
                'title': _('Too many CSS requests'),
                'description': _(
                    'This page has %(total_css_files)d CSS request '
                    '(%(over_limit)d over limit). Having too many '
                    'requests impose a tax in the browser due to handshakes.'),
                'category': _('Performance'),
                'generic_description': _(
                    'Pages with too many CSS requests aren\'t good to '
                    'performance. Overflowing this limit of requests can '
                    'impose a tax in the browser due to handshakes. This '
                    'limit of requests per page is configurable.'
                ),
                'unit': 'number'
            },
            'total.size.css': {
                'title': _('CSS size in kb is too big'),
                'description': _(
                    'There\'s %.2fkb of CSS in this page and that '
                    'adds up to more download time slowing down the '
                    'page rendering to the user.'),
                'category': _('Performance'),
                'generic_description': _(
                    'Pages with too big CSS files aren\'t good to performance. '
                    'Having this limit of file sizes overflow adds up to more '
                    'download time slowing down the page rendering to the user. '
                    'This limit of requests per page is configurable.'
                ),
                'unit': 'number'
            },
        }

    @classmethod
    def get_default_violations_values(cls, config):
        return {
            'total.requests.css': {
                'value': config.MAX_CSS_REQUESTS_PER_PAGE,
                'description': config.get_description('MAX_CSS_REQUESTS_PER_PAGE')
            },
            'total.size.css': {
                'value': config.MAX_CSS_KB_PER_PAGE_AFTER_GZIP,
                'description': config.get_description('MAX_CSS_KB_PER_PAGE_AFTER_GZIP')
            }
        }

    def validate(self):
        max_requests_css = self.get_violation_pref('total.requests.css')

        max_size_css_gzipped = self.get_violation_pref('total.size.css')

        total_css_files = self.get_total_requests_css()
        total_size_gzip = self.get_total_size_css_gzipped()

        over_limit = total_css_files - max_requests_css

        if total_css_files > max_requests_css:
            self.add_violation(
                key='total.requests.css',
                value={
                    'total_css_files': total_css_files,
                    'over_limit': over_limit
                },
                points=5 * over_limit
            )

        if total_size_gzip > max_size_css_gzipped:
            self.add_violation(
                key='total.size.css',
                value=total_size_gzip,
                points=int(total_size_gzip - max_size_css_gzipped)
            )

    def get_css_requests(self):
        return self.review.data.get('page.css', None)

    def get_total_requests_css(self):
        return self.review.data.get('total.requests.css', 0)

    def get_total_size_css_gzipped(self):
        return self.review.data.get('total.size.css.gzipped', 0)



"""added index for last_ping

Revision ID: 379d301c24ed
Revises: 2932df901655
Create Date: 2014-04-08 10:07:57.167665

"""

# revision identifiers, used by Alembic.
revision = '379d301c24ed'
down_revision = '2932df901655'

from alembic import op


def upgrade():
    op.create_index('idx_last_ping', 'workers', ['last_ping'])


def downgrade():
    op.drop_index('idx_last_ping', 'workers')



"""requests: create completed_date column

Revision ID: 556bb65b2a84
Revises: 59ceb7a18e80
Create Date: 2014-01-27 13:43:03.762366

"""

# revision identifiers, used by Alembic.
revision = '556bb65b2a84'
down_revision = '59ceb7a18e80'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        'requests',
        sa.Column('completed_date', sa.Date, nullable=False)
    )


def downgrade():
    op.drop_column('requests', 'completed_date')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.utils import is_valid
from holmes.validators.base import Validator
from holmes.utils import _


class DomainCanonicalizationValidator(Validator):

    @classmethod
    def get_no_301_parsed_value(cls, value):
        return {
            'effective_url': value['effective_url'],
            'location': value['headers']['Location'],
            'status_code': value['status_code']
        }

    @classmethod
    def get_violation_definitions(cls):
        return {
            'page.canonicalization.different_endpoints': {
                'title': _('Canonical URLs have different endpoints'),
                'description': _(
                    'This page have the canonical url`s "%(no_www_url)s" and '
                    '"%(www_url)s" with different endpoints. This both url`s '
                    'should point to the same location. '
                    'Not fixing this may cause search engines to be unsure as '
                    'to which URL is the correct one to index.'
                ),
                'category': _('SEO'),
                'generic_description': _(
                    'Canonical URLs is a couple of URLs with and without '
                    'the \'www\' prefix. For example: site.com and www.site.com '
                    'are canonical URLs. This violation is about a root page with '
                    'your canonical URLs pointing to different endpoints. This '
                    'behavior is down ranked in Search Engines indexing.'
                )
            },
            'page.canonicalization.no_301_redirect': {
                'title': _('Canonical URLs have a non 301 redirect'),
                'description': _(
                    'This Canonical url "%(effective_url)s" is redirecting to '
                    '"%(location)s" with a different than 301 status code: '
                    '%(status_code)s. This may cause search engines to be '
                    'unsure to where the URL is being redirected.'),
                'value_parser': cls.get_no_301_parsed_value,
                'category': _('SEO'),
                'generic_description': _(
                    'Canonical URLs is a couple of URLs with and without '
                    'the \'www\' prefix. For example: site.com and www.site.com '
                    'are canonical URLs. Canonical URLs should point to the '
                    'same points. This violation is about a redirection of one '
                    'of this URLs maked by a non 301 redirection. Redirections '
                    'different than a 301 type may cause search engines to be '
                    'unsure to where the URL is being redirected.'
                )
            }
        }

    def get_canonical_urls(self):
        url = self.reviewer.page_url
        parsed_url = is_valid(url)
        scheme_url = parsed_url.scheme
        domain_url = parsed_url.netloc

        if domain_url.startswith('www.'):
            www_url = url.rstrip('/')
            no_www_url = '{}://{}'.format(scheme_url, domain_url[4:])
        else:
            no_www_url = url.rstrip('/')
            www_url = '{}://www.{}'.format(scheme_url, domain_url)

        return {'www_url': www_url, 'no_www_url': no_www_url}

    def validate(self):
        if not self.reviewer.is_root():
            return

        canonical_urls = self.get_canonical_urls()

        self.async_get(canonical_urls['www_url'],
                       self.handle_www_url_res,
                       follow_redirects=False)
        self.async_get(canonical_urls['no_www_url'],
                       self.handle_no_www_url_res,
                       follow_redirects=False)
        self.reviewer.wait_for_async_requests()

        if not self.has_same_effective_urls():
            self.add_violation(
                key='page.canonicalization.different_endpoints',
                value=canonical_urls,
                points=50
            )
        else:
            redirect_error = self.has_redirect_violation()
            if redirect_error:
                self.add_violation(
                    key='page.canonicalization.no_301_redirect',
                    value=redirect_error,
                    points=50
                )

    def handle_www_url_res(self, url, response):
        self.www_url_res = response

    def handle_no_www_url_res(self, url, response):
        self.no_www_url_res = response

    def has_same_effective_urls(self):
        def target_url(res):
            if res.status_code != 200 and 'Location' in res.headers:
                return res.headers['Location'].rstrip('/')
            else:
                return res.effective_url.rstrip('/')

        return target_url(self.www_url_res) == target_url(self.no_www_url_res)

    def has_redirect_violation(self):
        def test(res):
            return res if (
                'Location' in res.headers
                and res.headers['Location'].rstrip('/')
                    != res.effective_url.rstrip('/')
                and res.status_code != 301
            ) else None

        return test(self.www_url_res) or test(self.no_www_url_res)



"""add active,domain idx to reviews

Revision ID: 1d98f72f9d10
Revises: 17346cf564bc
Create Date: 2014-03-12 11:59:05.795869

"""

# revision identifiers, used by Alembic.
revision = '1d98f72f9d10'
down_revision = '17346cf564bc'

from alembic import op


def upgrade():
    op.create_index('idx_is_active_domain', 'reviews', ['is_active', 'domain_id'])


def downgrade():
    op.drop_index('idx_is_active_domain', 'reviews')



"""create requests table

Revision ID: 59ceb7a18e80
Revises: 49c5cb27e7f6
Create Date: 2014-01-24 18:48:22.491856

"""

# revision identifiers, used by Alembic.
revision = '59ceb7a18e80'
down_revision = '49c5cb27e7f6'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'requests',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('domain_name', sa.String(120), nullable=False),
        sa.Column('url', sa.Text(), nullable=False),
        sa.Column('effective_url', sa.Text(), nullable=False),
        sa.Column('status_code', sa.Integer, nullable=False),
        sa.Column('response_time', sa.Float, nullable=False)
    )


def downgrade():
    op.drop_table('requests')



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sqlalchemy as sa
from sqlalchemy.orm import relationship
from sqlalchemy import func, distinct

from holmes.models import Base


class Domain(Base):
    __tablename__ = "domains"

    id = sa.Column(sa.Integer, primary_key=True)
    url = sa.Column('url', sa.String(2000), nullable=False)
    url_hash = sa.Column('url_hash', sa.String(128), nullable=False)
    name = sa.Column('name', sa.String(2000), nullable=False)

    is_active = sa.Column('is_active', sa.Boolean, default=True, nullable=False)

    pages = relationship("Page", backref="domain")
    reviews = relationship("Review", backref="domain")
    violations = relationship("Violation", backref="domain")
    violations_prefs = relationship("DomainsViolationsPrefs", backref="domain")

    def to_dict(self):
        return {
            "url": self.url,
            "name": self.name
        }

    @classmethod
    def get_pages_per_domain(cls, db):
        from holmes.models import Page
        return dict(db.query(Page.domain_id, sa.func.count(Page.id)).group_by(Page.domain_id).all())

    def get_homepage(self, db):
        from holmes.models import Page

        return db.query(Page).filter(Page.url == self.url).first()

    def get_page_count(self, db):
        from holmes.models import Page
        return db.query(func.count(Page.id)).filter(Page.domain_id == self.id).scalar()

    @classmethod
    def get_violations_per_domain(cls, db):
        from holmes.models import Review, Violation

        violations = db \
            .query(Review.domain_id, sa.func.count(Violation.id).label('count')) \
            .filter(Violation.review_id == Review.id) \
            .filter(Review.is_active == True) \
            .group_by(Review.domain_id) \
            .all()

        domains = {}
        for domain in violations:
            domains[domain.domain_id] = domain.count

        return domains

    def get_violation_data(self, db):
        from holmes.models import Violation

        result = db.query(sa.func.count(Violation.id).label('count')) \
            .filter(
                Violation.domain_id == self.id,
                Violation.review_is_active == True) \
            .one()

        return (
            result.count
        )

    def get_violations_per_day(self, db):
        from holmes.models import Review, Violation  # Prevent circular dependency

        violations = db \
            .query(
                sa.func.year(Review.completed_date).label('year'),
                sa.func.month(Review.completed_date).label('month'),
                sa.func.day(Review.completed_date).label('day'),
                sa.func.count(Violation.id).label('violation_count'),
                sa.func.sum(Violation.points).label('violation_points')
            ).join(
                Domain, Domain.id == Review.domain_id
            ).join(
                Violation, Violation.review_id == Review.id
            ).filter(Review.is_complete == True).filter(Review.domain_id == self.id) \
            .group_by(
                sa.func.year(Review.completed_date),
                sa.func.month(Review.completed_date),
                sa.func.day(Review.completed_date),
            ) \
            .order_by(Review.completed_date) \
            .all()

        result = []

        for day in violations:
            dt = "%d-%d-%d" % (day.year, day.month, day.day)
            result.append({
                "completedAt": dt,
                "violation_count": int(day.violation_count),
                "violation_points": int(day.violation_points)
            })

        return result

    def get_active_reviews(self, db, page_filter=None, current_page=1, page_size=10):
        from holmes.models import Page  # Prevent circular dependency

        lower_bound = (current_page - 1) * page_size
        upper_bound = lower_bound + page_size

        items_query = db \
            .query(
                Page.url, Page.uuid, Page.last_review_date,
                Page.last_review_uuid, Page.violations_count
            ) \
            .filter(Page.last_review_date != None) \
            .filter(Page.domain == self)

        if page_filter:
            items_query = items_query.filter(Page.url.like('%s/%s%%' % (self.url, page_filter)))

        items = items_query.order_by('violations_count desc')[lower_bound:upper_bound]

        return items

    @classmethod
    def get_domain_by_name(self, domain_name, db):
        return db.query(Domain).filter(Domain.name == domain_name).first()

    def get_active_review_count(self, db, page_filter=None):
        from holmes.models import Review, Page

        query = db.query(func.count(distinct(Review.page_id)))  # See #111

        if page_filter:
            query = query.join(Page, Page.id == Review.page_id) \
                .filter(Page.url.like('%s/%s%%' % (self.url, page_filter)))

        query = query.filter(Review.is_active == True, Review.domain_id == self.id)

        return query.scalar()

    @classmethod
    def get_all_domains(cls, db):
        return db.query(Domain).all()

    @classmethod
    def get_domain_names(cls, db):
        return [item.name for item in db.query(Domain.name).all()]

    def get_sample_requests(self, db, status_code_filter, limit=20000):
        from holmes.models import Request

        query = db \
            .query(
                Request.id,
                Request.response_time.label('response_time')
            ) \
            .filter(Request.domain_name == self.name)

        query = status_code_filter(query)

        return query \
            .order_by(Request.id.desc()) \
            .limit(limit) \
            .subquery()

    def get_good_request_count(self, db):
        from holmes.models import Request

        sample_query = self.get_sample_requests(
            db,
            lambda query: query.filter(Request.status_code < 400)
        )

        return db \
            .query(func.count(sample_query.columns.id)) \
            .scalar()

    def get_bad_request_count(self, db):
        from holmes.models import Request

        sample_query = self.get_sample_requests(
            db,
            lambda query: query.filter(Request.status_code > 399)
        )

        return db \
            .query(func.count(sample_query.columns.id)) \
            .scalar()

    def get_response_time_avg(self, db):
        from holmes.models import Request

        sample_query = self.get_sample_requests(
            db,
            lambda query: query.filter(Request.status_code < 400)
        )

        time_avg = db \
            .query(func.avg(sample_query.columns.response_time)) \
            .scalar() or 0

        return round(time_avg, 3)

    @classmethod
    def get_domains_details(cls, db):
        domains = db.query(Domain).order_by(Domain.name.asc()).all()

        if not domains:
            return []

        result = []

        for domain in domains:
            page_count = domain.get_page_count(db)
            review_count = domain.get_active_review_count(db)
            violation_count = domain.get_violation_data(db)
            good_request_count = domain.get_good_request_count(db)
            bad_request_count = domain.get_bad_request_count(db)
            response_time_avg = domain.get_response_time_avg(db)

            if page_count > 0:
                review_percentage = round(float(review_count) / page_count * 100, 2)
            else:
                review_percentage = 0

            total_request_count = good_request_count + bad_request_count
            if total_request_count > 0:
                error_percentage = round(float(bad_request_count) / total_request_count * 100, 2)
            else:
                error_percentage = 0

            result.append({
                "id": domain.id,
                "url": domain.url,
                "name": domain.name,
                "violationCount": violation_count,
                "pageCount": page_count,
                "reviewCount": review_count,
                "reviewPercentage": review_percentage,
                "errorPercentage": error_percentage,
                "is_active": domain.is_active,
                "averageResponseTime": response_time_avg,
            })

        return result

    @classmethod
    def get_active_domains(cls, db):
        return db.query(Domain).filter(Domain.is_active == True).all()



"""Added worker uuid as unique

Revision ID: 1c9004f0ab21
Revises: 7f4a3b8c55d
Create Date: 2014-03-31 18:37:10.921215

"""

# revision identifiers, used by Alembic.
revision = '1c9004f0ab21'
down_revision = '7f4a3b8c55d'

from alembic import op


def upgrade():
    op.create_unique_constraint('uk_uuid', 'workers', ['uuid'])


def downgrade():
    op.drop_constraint('uk_uuid', 'workers', type_='unique')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from holmes.validators.base import Validator
from holmes.utils import _


class ImageRequestsValidator(Validator):

    @classmethod
    def get_broken_images_parsed_values(cls, value):
        return {'images': ', '.join([
            '<a href="%s" target="_blank">Link #%s</a>' % (url, index)
            for index, url in enumerate(value)
        ])}

    @classmethod
    def get_single_image_size_parsed_value(cls, value):
        if 'over_max_size' in value:
            data = []
            for url, size_img in value['over_max_size']:
                name = url.rsplit('/', 1)[-1]
                desc = ' (%dkb)' % (size_img)
                data.append(
                    '<a href="%s" target="_blank">%s</a>%s' % (url, name, desc)
                )

            return {
                'limit': value['limit'], 'images': ', '.join(data)
            }

    @classmethod
    def get_violation_definitions(cls):
        return {
            'broken.img': {
                'title': _('Image not found'),
                'description': _(
                    'The image(s) in "%(images)s" could not be found or '
                    'took more than 10 seconds to load.'),
                'value_parser': cls.get_broken_images_parsed_values,
                'category': _('HTTP'),
                'generic_description': _(
                    'Image tags without a valid image resource. '
                    'The pointed resource can be a Client or a '
                    'Server Error, they can be loaded too slow or, '
                    'in most cases, means a not founded image.'
                )
            },
            'single.size.img': {
                'title': _('Single image size in kb is too big'),
                'description': _(
                    'Some images are above the expected limit '
                    '(%(limit)dkb): %(images)s'
                ),
                'value_parser': cls.get_single_image_size_parsed_value,
                'category': _('Performance'),
                'generic_description': _(
                    'Images with a too big size is very bad to site '
                    'performance. This limits are configurable in '
                    'Holmes configuration.'
                ),
                'unit': 'number'
            },
            'total.requests.img': {
                'title': _('Too many image requests'),
                'description': _(
                    'This page has %(total)d image requests (%(limit)d over '
                    'limit). Having too many requests impose a tax in the '
                    'browser due to handshakes.'),
                'category': _('Performance'),
                'generic_description': _(
                    'A site with too many images requests per page can '
                    'deacrease the page load speed and performance. This '
                    'limits are configurable in Holmes configuration.'
                ),
                'unit': 'number'
            },
            'total.size.img': {
                'title': _('Total image size in kb is too big'),
                'description': _(
                    'There`s %.2fkb of images in this page and that '
                    'adds up to more download time slowing down the page '
                    'rendering to the user.'),
                'category': _('Performance'),
                'generic_description': _(
                    'A site with a too big total image size per page can '
                    'decrease the page load speed and performance. This '
                    'limits are configurable in Holmes configuration.'
                ),
                'unit': 'number'
            }
        }

    @classmethod
    def get_default_violations_values(cls, config):
        return {
            'single.size.img':  {
                'value': config.MAX_KB_SINGLE_IMAGE,
                'description': config.get_description('MAX_KB_SINGLE_IMAGE')
            },
            'total.requests.img': {
                'value': config.MAX_IMG_REQUESTS_PER_PAGE,
                'description': config.get_description('MAX_IMG_REQUESTS_PER_PAGE')
            },
            'total.size.img': {
                'value': config.MAX_IMG_KB_PER_PAGE,
                'description': config.get_description('MAX_IMG_KB_PER_PAGE')
            }
        }

    def validate(self):
        max_single_size_img = self.get_violation_pref('single.size.img')

        max_total_requests_img = self.get_violation_pref('total.requests.img')

        max_total_size_img = self.get_violation_pref('total.size.img')

        img_files = self.get_images()
        total_size = self.get_images_size()

        broken_imgs = set()

        over_max_size = set()

        for url, response in img_files:

            if response.status_code > 399:
                broken_imgs.add(url)

            if response.text is not None:
                size_img = len(response.text) / 1024.0

                if size_img > max_single_size_img:
                    over_max_size.add((url, size_img))

        if broken_imgs:
            self.add_violation(
                key='broken.img',
                value=broken_imgs,
                points=50
            )

        if over_max_size:
            points = 0
            for url, size_img in over_max_size:
                size = size_img - max_single_size_img
                points += size

            self.add_violation(
                key='single.size.img',
                value={
                    'limit': max_single_size_img,
                    'over_max_size': over_max_size
                },
                points=points
            )

        if len(img_files) > max_total_requests_img:
            self.add_violation(
                key='total.requests.img',
                value={
                    'total': len(img_files),
                    'limit': len(img_files) - max_total_requests_img
                },
                points=5 * (len(img_files) - max_total_requests_img)
            )

        if total_size > max_total_size_img:
            self.add_violation(
                key='total.size.img',
                value=total_size,
                points=int(total_size - max_total_size_img)
            )

    def get_images(self):
        return self.review.data.get('page.images', None)

    def get_images_size(self):
        return self.review.data.get('total.size.img', 0)



"""create facts table

Revision ID: 52988f72b077
Revises: 5048316f2933
Create Date: 2013-11-13 11:15:00.873936

"""

# revision identifiers, used by Alembic.
revision = '52988f72b077'
down_revision = '5048316f2933'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'facts',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('title', sa.String(2000), nullable=False),
        sa.Column('key', sa.String(2000), nullable=False),
        sa.Column('unit', sa.String(2000), nullable=False),
        sa.Column('value', sa.String(4000), nullable=False),
        sa.Column('review_id', sa.Integer),
    )

    op.create_foreign_key(
        "fk_fact_review", "facts",
        "reviews", ["review_id"], ["id"]
    )


def downgrade():
    op.drop_constraint('fk_fact_review', 'facts', type_="foreignkey")
    op.drop_table('facts')



#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging
from datetime import datetime, timedelta

from ujson import dumps
from tornado.web import RequestHandler

from holmes import __version__
from holmes.models import User
import holmes.utils as utils


class BaseHandler(RequestHandler):

    @property
    def config(self):
        return self.application.config

    def initialize(self, *args, **kw):
        self.is_public = kw.pop('is_public', False)
        super(BaseHandler, self).initialize(*args, **kw)

        locale = self.get_browser_locale()
        self._ = utils.install_i18n(locale.code)
        self.jwt = utils.Jwt(self.config.SECRET_KEY)

    def get_authenticated_user(self):
        authenticated, payload = self.is_authenticated()
        if authenticated:
            user_email = payload['sub']
            user = User.by_email(user_email, self.db)
            return user
        else:
            return None

    def validate_superuser(self):
        user = self.get_authenticated_user()
        if not user or not user.is_superuser:
            self.set_unauthorized()
            return False
        return True

    def set_unauthorized(self):
        self.set_status(401)
        self.write('Unauthorized')
        self.finish()

    def renew_authentication(self, payload):
        payload.update(dict(
            iat=datetime.utcnow(),
            exp=datetime.utcnow() + timedelta(
                seconds=self.config.SESSION_EXPIRATION
            )
        ))
        token = self.jwt.encode(payload)
        self.set_cookie('HOLMES_AUTH_TOKEN', token)

    def is_authenticated(self):
        return self.jwt.try_to_decode(self.get_cookie('HOLMES_AUTH_TOKEN'))

    def authenticate_request(self):
        authenticated, payload = self.is_authenticated()
        if authenticated:
            self.renew_authentication(payload)
        else:
            self.set_unauthorized()

    def prepare(self):
        if self.request.method != 'OPTIONS' and not self.is_public:
            self.authenticate_request()

    def log_exception(self, typ, value, tb):
        for handler in self.application.error_handlers:
            handler.handle_exception(
                typ, value, tb, extra={
                    'url': self.request.full_url(),
                    'ip': self.request.remote_ip,
                    'holmes-version': __version__
                }
            )

        super(BaseHandler, self).log_exception(typ, value, tb)

    def on_finish(self):
        if self.application.config.COMMIT_ON_REQUEST_END:
            if self.get_status() > 399:
                logging.debug('ROLLING BACK TRANSACTION')
                self.db.rollback()
            else:
                logging.debug('COMMITTING TRANSACTION')
                self.db.flush()
                self.db.commit()
                self.application.event_bus.flush()

    def options(self, *args):
        self.set_status(200)
        self.finish()

    def set_default_headers(self):
        self.set_header(
            'Access-Control-Allow-Origin',
            self.application.config.HOLMES_WEB_URL
        )
        self.set_header('Access-Control-Allow-Credentials', 'true')
        self.set_header(
            'Access-Control-Allow-Methods', 'GET,PUT,POST,DELETE,OPTIONS'
        )
        self.set_header('Access-Control-Allow-Headers', 'Accept, Content-Type')

    def write_json(self, obj):
        self.set_header("Content-Type", "application/json")
        self.write(dumps(obj))

    @property
    def cache(self):
        return self.application.cache

    @property
    def db(self):
        return self.application.db

    @property
    def girl(self):
        return self.application.girl



#!/usr/bin/python
# -*- coding: utf-8 -*-

from ujson import loads
from tornado.gen import coroutine

from holmes.models import Limiter
from holmes.handlers import BaseHandler


class LimiterHandler(BaseHandler):
    @coroutine
    def options(self, id=None):
        super(LimiterHandler, self).options()

    @coroutine
    def get(self):

        limiters = Limiter.get_all(
            self.db, domain_filter=self.get_argument('domain_filter', None)
        )

        result = []

        for limit in limiters:
            current_value = yield self.cache.get_limit_usage(limit.url) or 0

            percentage = 0
            if limit.value > 0:
                percentage = float(current_value) / limit.value

            result.append({
                'id': limit.id,
                'url': limit.url,
                'currentValue': current_value,
                'maxValue': limit.value or 0,
                'concurrentRequestsPercentage': percentage
            })

        self.write_json(result)

    @coroutine
    def post(self):
        if not self.validate_superuser():
            return

        post_data = loads(self.request.body)
        url = post_data.get('url', None)
        connections = self.config.DEFAULT_NUMBER_OF_CONCURRENT_CONNECTIONS
        value = post_data.get('value', connections)

        if not url and not value:
            self.set_status(400)
            self.write_json({
                'reason': 'Not url or value',
                'description': self._('Not url or value')
            })
            return

        result = Limiter.add_or_update_limiter(self.db, url, value)

        yield self.cache.remove_domain_limiters_key()

        self.write_json(result)

    @coroutine
    def delete(self, limiter_id=None):
        if not self.validate_superuser():
            return

        if not limiter_id:
            self.set_status(400)
            self.write_json({'reason': 'Invalid data', 'description': self._('Invalid data')})
            return

        limiter = Limiter.by_id(limiter_id, self.db)

        if not limiter or not limiter.id:
            self.set_status(404)
            self.write_json({'reason': 'Not Found', 'description': self._('Not Found')})
            return

        Limiter.delete(limiter.id, self.db)

        yield self.cache.remove_domain_limiters_key()

        self.set_status(204)



"""Page url index

Revision ID: 3a3aa7483604
Revises: 4260bbc20569
Create Date: 2013-11-27 15:58:18.367398

"""

# revision identifiers, used by Alembic.
revision = '3a3aa7483604'
down_revision = '4260bbc20569'

from alembic import op


def upgrade():
    op.execute('CREATE INDEX idx_pages_url on pages (url(220))')


def downgrade():
    op.drop_index('idx_pages_url', 'pages')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from uuid import UUID

from tornado import gen
from ujson import loads

from holmes.models import Page, Review
from holmes.handlers import BaseHandler


class PageHandler(BaseHandler):
    def get(self, uuid=''):
        uuid = UUID(uuid)

        page = Page.by_uuid(uuid, self.db)

        if not page:
            self.set_status(404, self._('Page UUID [%s] not found') % uuid)
            return

        page_json = {
            "uuid": str(page.uuid),
            "url": page.url
        }

        self.write(page_json)

    @gen.coroutine
    def post(self):
        post_data = loads(self.request.body)
        url = post_data['url']
        score = float(post_data.get('score', self.application.config.DEFAULT_PAGE_SCORE))

        result = yield Page.add_page(
            self.db,
            self.application.cache,
            url,
            score,
            self.application.http_client.fetch,
            self.application.event_bus.publish,
            self.application.config,
            self.application.girl,
            self.application.default_violations_values,
            self.application.violation_definitions
        )

        created, url, result = result

        if not created and result['reason'] == 'invalid_url':
            self.set_status(400, self._('Invalid url [%s]') % url)
            self.write_json({
                'reason': 'invalid_url',
                'url': url,
                'status': result['status'],
                'details': result['details']
            })
            return

        if not created and result['reason'] == 'redirect':
            self.set_status(400, self._('Supplied URL is a redirect [%s]') % url)
            self.write_json({
                'reason': 'redirect',
                'url': url,
                'effectiveUrl': result['effectiveUrl']
            })
            return

        yield self.application.cache.add_next_job_bucket(result, url)

        self.write(str(result))
        self.finish()


class PageReviewsHandler(BaseHandler):

    def get(self, uuid='', limit=10):
        uuid = UUID(uuid)

        page = Page.by_uuid(uuid, self.db)

        if not page:
            self.set_status(404, self._('Page UUID [%s] not found') % uuid)
            return

        reviews = self.db.query(Review) \
            .filter(Review.page == page) \
            .filter(Review.is_complete == True) \
            .order_by(Review.completed_date.desc())[:limit]

        result = []
        for review in reviews:
            result.append({
                'uuid': str(review.uuid),
                'completedAt': review.completed_date,
                'violationCount': review.violation_count
            })

        self.write_json(result)


class PageViolationsPerDayHandler(BaseHandler):

    def get(self, uuid):
        page = Page.by_uuid(uuid, self.db)

        if not page:
            self.set_status(404, self._('Page UUID [%s] not found') % uuid)
            return

        violations_per_day = page.get_violations_per_day(self.db)

        page_json = {
            "violations": violations_per_day
        }

        self.write_json(page_json)


class NextJobHandler(BaseHandler):

    @gen.coroutine
    def get(self):
        current_page=int(self.get_argument('current_page', 1))
        page_size=int(self.get_argument('page_size', 10))

        next_job_list = yield self.cache.get_next_job_list(current_page, page_size)

        jobs = []
        for idx, data in enumerate(next_job_list):
            page = loads(data)
            page['num'] = idx + 1 + (current_page - 1) * page_size
            jobs.append(page)

        self.write_json({'pages': jobs})



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from holmes import __version__

tests_require = [
    'mock',
    'nose',
    'coverage',
    'yanc',
    'preggy',
    'tox',
    'ipdb',
    'coveralls',
    'factory_boy',
    'sqltap',
    'sphinx',
    'honcho',
]

setup(
    name='holmes',
    version=__version__,
    description='Holmes is a service to investigate your website health.',
    long_description='''
Holmes is a service to investigate your website health.
''',
    keywords='seo health web',
    author='Globo.com',
    author_email='appdev@corp.globo.com',
    url='',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'derpconf>=0.6.0,<0.7.0',
        'tornado>=3.2.2,<3.3.0',
        'cow-framework>=0.8.0,<0.9.0',
        'ujson>=1.33,<1.34.0',
        'requests>=2.2.1,<2.3.0',
        'lxml>=3.3.3,<3.4.0',
        'cssselect>=0.9.1,<0.10.0',
        'sheep>=0.3.10,<0.4.0',
        'pycurl>=7.19.0,<7.20.0',
        'alembic>=0.6.3,<0.7.0',
        'mysql-python>=1.2.5,<1.3.0',
        'six>=1.6.1,<1.7.0',
        'octopus-http>=0.6.3,<0.7.0',
        'redis>=2.9.1,<2.11.0',
        'toredis>=0.1.2,<0.2.0',
        'raven>=4.1.1,<4.2.0',
        'rotunicode>=1.0.1,<1.1.0',
        'materialgirl>=0.5.3,<0.6.0',
        'pyelasticsearch>=0.6.1,<0.7.0',
        'tornadoes>=2.1.0,<2.2.0',
        'Babel>=1.3,<1.4',
        'holmesalf>=0.1.2,<0.2.0',
        'PyJWT>=0.2.1,<0.3.0'
    ],
    extras_require={
        'tests': tests_require,
    },
    entry_points={
        'console_scripts': [
            'holmes-api=holmes.server:main',
            'holmes-worker=holmes.worker:main',
            'holmes-material=holmes.material:main',
            'holmes-search=holmes.search:main',
        ],
    },
)



"""Key name must be unique

Revision ID: cc4091b07cb
Revises: 2f69133b476d
Create Date: 2013-12-12 11:23:55.314017

"""

# revision identifiers, used by Alembic.
revision = 'cc4091b07cb'
down_revision = '2f69133b476d'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.alter_column('keys', 'name', type_=sa.String(255))
    op.create_unique_constraint("uk_name", "keys", ["name"])


def downgrade():
    op.drop_constraint('uk_name', 'keys', type_='unique')
    op.alter_column('keys', 'name', type_=sa.String(2000))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import logging

from derpconf.config import ConfigurationError

from holmes.config import Config
from holmes.utils import load_classes
from holmes.search_providers import SearchProvider


def main():
    parser = SearchProvider.argparser()
    args = parser.parse_args()
    try:
        config = Config()
        if args.conf:
            config = config.load(args.conf[0])
        search_providers = load_classes(default=[config['SEARCH_PROVIDER']])
        if isinstance(search_providers, list) and len(search_providers) == 1:
            search_provider = search_providers.pop()
            search_provider.main()
        else:
            logging.error('Could not instantiate search provider!')
            sys.exit(1)
    except ConfigurationError:
        logging.error('Could not load config! Use --conf conf_file')
        sys.exit(1)
    except KeyError:
        logging.error('Could not parse config! Check it\'s contents')
        sys.exit(1)

if __name__ == '__main__':
    main()



"""Review table indexes for active reviews

Revision ID: 5edcaddd6c9
Revises: 10e3ffb42fc9
Create Date: 2013-11-27 15:28:20.189260

"""

# revision identifiers, used by Alembic.
revision = '5edcaddd6c9'
down_revision = '10e3ffb42fc9'

from alembic import op


def upgrade():
    op.create_index('idx_reviews_is_active', 'reviews', ['is_active'])
    op.create_index('idx_reviews_completed_date', 'reviews', ['completed_date'])


def downgrade():
    op.drop_index('idx_reviews_is_active', 'reviews')
    op.drop_index('idx_reviews_completed_date', 'reviews')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from holmes.search_providers import SearchProvider
from holmes.models.review import Review

from tornado.concurrent import return_future


class NoExternalSearchProvider(SearchProvider):
    def __init__(self, config, db, authnz_wrapper=None, io_loop=None):
        self.db = db

    def index_review(self, review):
        pass

    @return_future
    def get_by_violation_key_name(self, key_id, current_page=1, page_size=10, domain=None, page_filter=None, callback=None):
        reviews = Review.get_by_violation_key_name(
            db=self.db,
            key_id=key_id,
            current_page=current_page,
            page_size=page_size,
            domain_filter=domain.name if domain else None,
            page_filter=page_filter,
        )

        reviews_data = []
        for item in reviews:
            reviews_data.append({
                'uuid': item.review_uuid,
                'page': {
                    'uuid': item.page_uuid,
                    'url': item.url,
                    'completedAt': item.completed_date
                },
                'domain': item.domain_name,
            })

        callback({
            'reviews': reviews_data
        })

    @return_future
    def get_domain_active_reviews(self, domain, current_page=1, page_size=10, page_filter=None, callback=None):
        reviews = domain.get_active_reviews(
            db=self.db,
            page_filter=page_filter,
            current_page=current_page,
            page_size=page_size,
        )

        pages = []

        for page in reviews:
            pages.append({
                'url': page.url,
                'uuid': str(page.uuid),
                'violationCount': page.violations_count,
                'completedAt': page.last_review_date,
                'reviewId': str(page.last_review_uuid)
            })

        callback({'pages': pages})

    @classmethod
    def new_instance(cls, config):
        return NoExternalSearchProvider(config)

    @classmethod
    def main(cls):
        pass

if __name__ == '__main__':
    NoExternalSearchProvider.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from tornado.gen import coroutine

from holmes.models import Domain, KeysCategory
from holmes.handlers import BaseHandler


class DomainsHandler(BaseHandler):

    @coroutine
    def get(self):
        domains = self.db.query(Domain).order_by(Domain.name.asc()).all()

        if not domains:
            self.write("[]")
            return

        result = []

        for domain in domains:
            result.append({
                "id": domain.id,
                "url": domain.url,
                "name": domain.name,
                "is_active": domain.is_active
            })

        self.write_json(result)


class DomainsFullDataHandler(BaseHandler):

    @coroutine
    def get(self):
        result = self.girl.get('domains_details') or []
        self.write_json(result)


class DomainDetailsHandler(BaseHandler):

    @coroutine
    def get(self, domain_name):
        domain = Domain.get_domain_by_name(domain_name, self.db)

        if not domain:
            self.set_status(404, self._('Domain %s not found') % domain_name)
            return

        result = self.girl.get('domains_details') or []
        data = next((l for l in result if l['name'] == domain_name), None)

        if not data:
            self.set_status(404, self._('Domain %s not found') % domain_name)
            return

        page_count = data.get('pageCount', 0)
        review_count = data.get('reviewCount', 0)
        violation_count = data.get('violationCount', 0)
        error_percentage = data.get('errorPercentage', 0)
        response_time_avg = data.get('averageResponseTime', 0)
        review_percentage = data.get('reviewPercentage', 0)

        domain_json = {
            "id": domain.id,
            "name": domain.name,
            "url": domain.url,
            "pageCount": page_count,
            "reviewCount": review_count,
            "violationCount": violation_count,
            "reviewPercentage": review_percentage,
            "is_active": domain.is_active,
            "errorPercentage": error_percentage,
            "averageResponseTime": response_time_avg,
            "homepageId": "",
            "homepageReviewId": "",
        }

        homepage = domain.get_homepage(self.db)

        if homepage:
            if homepage.uuid:
                domain_json["homepageId"] = str(homepage.uuid)
            if homepage.last_review_uuid:
                domain_json["homepageReviewId"] = str(homepage.last_review_uuid)

        self.write_json(domain_json)


class DomainViolationsPerDayHandler(BaseHandler):

    def get(self, domain_name):
        domain = Domain.get_domain_by_name(domain_name, self.db)

        if not domain:
            self.set_status(404, self._('Domain %s not found') % domain_name)
            return

        violations_per_day = domain.get_violations_per_day(self.db)

        domain_json = {
            "id": domain.id,
            "name": domain.name,
            "url": domain.url,
            "violations": violations_per_day
        }

        self.write_json(domain_json)


class DomainReviewsHandler(BaseHandler):

    @coroutine
    def get(self, domain_name):
        term = self.get_argument('term', None)
        current_page = int(self.get_argument('current_page', 1))
        page_size = int(self.get_argument('page_size', 10))

        domain = Domain.get_domain_by_name(domain_name, self.db)

        if not domain:
            self.set_status(404, self._('Domain %s not found') % domain_name)
            return

        reviews = yield self.application.search_provider.get_domain_active_reviews(
            domain=domain,
            current_page=current_page,
            page_size=page_size,
            page_filter=term,
        )

        if 'error' in reviews:
            self.set_status(reviews['error']['status_code'], reviews['error']['reason'])
            self.finish()
            return

        if 'reviewsCount' not in reviews:
            if not term:
                reviews['reviewsCount'] = yield self.cache.get_active_review_count(domain)
            else:
                reviews['reviewsCount'] = None

        self.write_json(reviews)
        self.finish()


class DomainGroupedViolationsHandler(BaseHandler):

    @coroutine
    def get(self, domain_name):
        domain = Domain.get_domain_by_name(domain_name, self.db)

        if not domain:
            self.set_status(404, self._('Domain %s not found') % domain_name)
            return

        violation_defs = self.application.violation_definitions

        grouped_violations = self.girl.get('violation_count_by_category_for_domains') or {}

        total = 0
        violations = []

        for item in grouped_violations.get(domain.id, []):
            key_name, key_category_id, count = item['key_name'], item['category_id'], item['violation_count']
            violations.append({
                'categoryId': key_category_id,
                'categoryName': self._(violation_defs[key_name]['category']),
                'count': count
            })
            total += count

        result = {
            "domainId": domain.id,
            'domainName': domain.name,
            'domainURL': domain.url,
            'total': total,
            'violations': violations
        }

        self.write_json(result)


class DomainTopCategoryViolationsHandler(BaseHandler):

    @coroutine
    def get(self, domain_name, key_category_id):
        domain = Domain.get_domain_by_name(domain_name, self.db)

        if not domain:
            self.set_status(404, self._('Domain %s not found') % domain_name)
            return

        key_category = KeysCategory.get_by_id(self.db, key_category_id)

        if not key_category:
            self.set_status(404, self._('Key category %s not found') % key_category_id)
            return

        violation_defs = self.application.violation_definitions

        top_violations = self.girl.get('top_violations_in_category_for_domains') or {}

        violations = []
        for top_violation in top_violations.get(domain_name, {}).get(key_category.id, []):
            violations.append({
                'title': self._(violation_defs[top_violation['key_name']]['title']),
                'count': top_violation['count'],
                'key': top_violation['key_name'],
            })

        result = {
            "domainId": domain.id,
            'domainName': domain.name,
            'domainURL': domain.url,
            'categoryId': key_category_id,
            'violations': violations
        }

        self.write_json(result)


class DomainsChangeStatusHandler(BaseHandler):

    @coroutine
    def post(self, domain_name):
        domain = Domain.get_domain_by_name(domain_name, self.db)

        if not domain:
            self.set_status(404, self._('Domain %s not found') % domain_name)
            return

        domain.is_active = not domain.is_active

        if not domain.is_active:
            yield self.cache.delete_limit_usage_by_domain(domain.url)
